package com.edgec.browserbackend.browser.task;

import com.edgec.browserbackend.account.domain.Account;
import com.edgec.browserbackend.account.domain.UserPrePaidBilling;
import com.edgec.browserbackend.account.repository.AccountRepository;
import com.edgec.browserbackend.account.repository.UserPrePaidBillingRepository;
import net.javacrumbs.shedlock.core.SchedulerLock;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.time.YearMonth;
import java.time.ZoneOffset;
import java.util.List;

@Component
public class PromotionTask {

    private static final Logger log = LoggerFactory.getLogger(PromotionTask.class);

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private UserPrePaidBillingRepository userPrePaidBillingRepository;

    @SchedulerLock(name = "transferExpiredScalingGroups", lockAtLeastForString = "PT1H", lockAtMostForString = "PT2H")
    @Scheduled(cron = "0 0 1 1 * ?")
    public void countCommission() {
        List<Account> accounts = accountRepository.findByParentIsNull();
        for (Account account : accounts) {
            String code = account.getPromotion().getCode();
            List<Account> promotes = accountRepository.findByPromotionCode(code);
            double totalCommission = 0;
            for (Account promote : promotes) {
                List<UserPrePaidBilling> userPrePaidBillings = userPrePaidBillingRepository.findByUsernameAndYearAndMonthAndPayMethodIsNot(promote.getName(),
                        YearMonth.now().minusMonths(1).getYear(),
                        YearMonth.now().minusMonths(1).getMonthValue(), 0);
                totalCommission += userPrePaidBillings.stream().mapToDouble(UserPrePaidBilling::getTotal).sum();
            }
            account.getPromotion().setCommission((int)totalCommission);
            accountRepository.save(account);
        }
    }
}
